#!/bin/bash -e

wd=$(pwd)
jobid=$(squeue --me | head -2 | tail -n1 | awk '{print $1}')


#
# Example assume allocation was created, e.g.:
# N=1 ; salloc -p standard-g  --threads-per-core 1 --exclusive -N $N --gpus $((N*8)) -t 4:00:00 --mem 0
#

module purge
module load CrayEnv
module load PrgEnv-gnu/8.4.0 gcc/10.3.0
module load craype-x86-trento
module load craype-accel-amd-gfx90a

module use /pfs/lustrep2/projappl/project_462000125/samantao-public/mymodules
module load rocm/5.6.1.lua

set -x

# Clone source
if [ ! -d $wd/pytorch-source ] ; then
    git clone -b v2.2.2 --recursive https://github.com/pytorch/pytorch $wd/pytorch-source 
    cd $wd/pytorch-source 
    git submodule sync
    git submodule update --init --recursive --jobs 0
fi

# Install pytorch dependencies if it doesn't exist already.
if [ ! -d $wd/miniconda3/envs/pytorch-from-source ] ; then
    # Miniconda should exist already
    source $wd/miniconda3/bin/activate base
    conda create -y -n pytorch-from-source python=3.10
    source $wd/miniconda3/bin/activate pytorch-from-source

    pip install -r $wd/pytorch-source/requirements.txt
    conda install -y cmake ninja

    # Let's use the system libstdc++
    rm -rf $wd/miniconda3/envs/pytorch-from-source/lib/libstdc++.so*
else
    source $wd/miniconda3/bin/activate pytorch-from-source
fi

# Build pytorch wheel file from source
if [ ! -f $wd/pytorch-source/dist/torch-*.whl ] ; then
    cd $wd/pytorch-source
    rm -rf build

    # Force our ROCM instalation that is not in a default path.
    grep -rl /opt/rocm | xargs sed -i "s#/opt/rocm#$ROCM_PATH#g" || true

    # Hipify source
    nice python3 tools/amd_build/build_amd.py

    # Build with debug symbols
    CC=$(which gcc) \
    CXX=$(which g++) \
    CMAKE_PREFIX_PATH=$CONDA_PREFIX:$CMAKE_PREFIX_PATH \
        LDFLAGS="-L$ROCM_PATH/deps -lstdc++ -ltinfo" \
        USE_KINETO=0 BUILD_TEST=0 \
        PYTORCH_ROCM_ARCH=gfx90a \
        RCCL_PATH=$ROCM_PATH/rccl \
        RCCL_DIR=$ROCM_PATH/rccl/lib/cmake \
        hip_DIR=${ROCM_PATH}/hip/cmake/ \
        REL_WITH_DEB_INFO=1 \
        nice python3 setup.py bdist_wheel

    pip install $wd/pytorch-source/dist/torch-*.whl
fi

# Make sure conda libs can be loaded.
# export LD_LIBRARY_PATH=$CONDA_PREFIX/lib:$LD_LIBRARY_PATH 

srun --jobid=$jobid -n1 --gpus 8 \
    python -c 'import torch; print("I have this many devices:", torch.cuda.device_count())'
